<?php

namespace Bixo\Collection\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Collection\Forms\Collections as CollectionsForm;
use Bixo\Collection\Http\Requests\CollectionsResourceRequest;
use Bixo\Collection\Http\Resources\CollectionsResource;
use Bixo\Collection\Http\Resources\CollectionsCollection;
use Bixo\Collection\Models\Collections;
use Bixo\Collection\Actions\CollectionsAction;
use Bixo\Collection\Actions\CollectionsActions;


/**
 * Resource controller class for collections.
 */
class CollectionsResourceController extends BaseController
{

    /**
     * Initialize collections resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = CollectionsForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.collection.modules'), 'collection', guard_url('collection'));
            return $next($request);
        });
    }

    /**
     * Display a list of collections.
     *
     * @return Response
     */
    public function index(CollectionsResourceRequest $request)
    {
        $request = $request->all();
        $page = CollectionsActions::run('paginate', $request);

        $data = new CollectionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('collection::collections.names'))
            ->view('collection::collections.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display collections.
     *
     * @param Request $request
     * @param Model   $collections
     *
     * @return Response
     */
    public function show(CollectionsResourceRequest $request, Collections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CollectionsResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('collection::collections.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('collection::collections.show')
            ->output();
    }

    /**
     * Show the form for creating a new collections.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CollectionsResourceRequest $request, Collections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CollectionsResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('collection::collections.name'))
            ->view('collection::collections.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new collections.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CollectionsResourceRequest $request, Collections $model)
    {
        try {
            $request = $request->all();
            $model = CollectionsAction::run('store', $model, $request);
            $data = new CollectionsResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('collection::collections.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('collection/collections/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/collection/collections'))
                ->redirect();
        }

    }

    /**
     * Show collections for editing.
     *
     * @param Request $request
     * @param Model   $collections
     *
     * @return Response
     */
    public function edit(CollectionsResourceRequest $request, Collections $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CollectionsResource($model);
        // return view('collection::collections.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('collection::collections.name'))
            ->view('collection::collections.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the collections.
     *
     * @param Request $request
     * @param Model   $collections
     *
     * @return Response
     */
    public function update(CollectionsResourceRequest $request, Collections $model)
    {
        try {
            $request = $request->all();
            $model = CollectionsAction::run('update', $model, $request);
            $data = new CollectionsResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('collection::collections.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('collection/collections/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('collection/collections/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the collections.
     *
     * @param Model   $collections
     *
     * @return Response
     */
    public function destroy(CollectionsResourceRequest $request, Collections $model)
    {
        try {

            $request = $request->all();
            $model = CollectionsAction::run('destroy', $model, $request);
            $data = new CollectionsResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('collection::collections.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('collection/collections/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('collection/collections/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
